@@ -6,7 +6,7 @@ use strict;
 
 use vars qw/$VERSION/;
 
-$VERSION = '0.44';
+$VERSION = '0.46';
 
 # Package to store unsigned big integers in decimal and do math with them
 
@@ -15,7 +15,7 @@ $VERSION = '0.44';
 # automatically at loading time to be the maximum possible value
 
 # todo:
-# - fully remove funky $# stuff (maybe)
+# - fully remove funky $# stuff in div() (maybe - that code scares me...)
 
 # USE_MUL: due to problems on certain os (os390, posix-bc) "* 1e-5" is used
 # instead of "/ 1e5" at some places, (marked with USE_MUL). Other platforms
@@ -36,7 +36,7 @@ $VERSION = '0.44';
 sub api_version () { 1; }
  
 # constants for easier life
-my ($MBASE,$BASE,$RBASE,$BASE_LEN,$MAX_VAL,$BASE_LEN_SMALL);
+my ($BASE,$BASE_LEN,$MBASE,$RBASE,$MAX_VAL,$BASE_LEN_SMALL);
 my ($AND_BITS,$XOR_BITS,$OR_BITS);
 my ($AND_MASK,$XOR_MASK,$OR_MASK);
 
@@ -94,7 +94,7 @@ sub _base_len
       }
     }
   return $BASE_LEN unless wantarray;
-  return ($BASE_LEN, $AND_BITS, $XOR_BITS, $OR_BITS, $BASE_LEN_SMALL, $MAX_VAL);
+  return ($BASE_LEN, $AND_BITS, $XOR_BITS, $OR_BITS, $BASE_LEN_SMALL, $MAX_VAL, $BASE);
   }
 
 sub _new
@@ -578,7 +578,25 @@ sub _div_use_mul
       # between 1 and MAX_VAL (e.g. one element) and rem is not wanted.
       if (!wantarray)
         {
-        $x->[0] = int($x->[-1] / $yorg->[-1]);
+        # fit's into one Perl scalar, so result can be computed directly
+        # cannot use int() here, because it rounds wrongly on some systems
+        #$x->[0] = int($x->[-1] / $yorg->[-1]);
+
+ 	# Due to chopping up the number into parts, the two first parts
+	# may have only one or two digits. So we use more from the second
+	# parts (it always has at least two parts) for more accuracy:
+        # Round to 8 digits, then truncate result to integer:
+	my $x0 = $x->[-1];
+	my $y0 = $yorg->[-1];
+	if (length ($x0) < $BASE_LEN)		# len($x0) == len($y0)!
+	  {
+          $x0 .= substr('0' x $BASE_LEN . $x->[-2], -$BASE_LEN, $BASE_LEN);
+          $x0 = substr($x0,0,$BASE_LEN);
+          $y0 .= substr('0' x $BASE_LEN . $yorg->[-2], -$BASE_LEN, $BASE_LEN);
+          $y0 = substr($y0,0,$BASE_LEN);
+	  }
+        $x->[0] = int ( sprintf ("%.8f", $x0 / $y0 ) );
+
         splice(@$x,1);			# keep single element
         return $x;
         }
@@ -775,7 +793,25 @@ sub _div_use_div
       # between 1 and MAX_VAL (e.g. one element) and rem is not wanted.
       if (!wantarray)
         {
-        $x->[0] = int($x->[-1] / $yorg->[-1]);
+        # fit's into one Perl scalar, so result can be computed directly
+        # cannot use int() here, because it rounds wrongly on some systems
+        #$x->[0] = int($x->[-1] / $yorg->[-1]);
+
+ 	# Due to chopping up the number into parts, the two first parts
+	# may have only one or two digits. So we use more from the second
+	# parts (it always has at least two parts) for more accuracy:
+        # Round to 8 digits, then truncate result to integer:
+	my $x0 = $x->[-1];
+	my $y0 = $yorg->[-1];
+	if (length ($x0) < $BASE_LEN)		# len($x0) == len($y0)!
+	  {
+          $x0 .= substr('0' x $BASE_LEN . $x->[-2], -$BASE_LEN, $BASE_LEN);
+          $x0 = substr($x0,0,$BASE_LEN);
+          $y0 .= substr('0' x $BASE_LEN . $yorg->[-2], -$BASE_LEN, $BASE_LEN);
+          $y0 = substr($y0,0,$BASE_LEN);
+	  }
+        $x->[0] = int ( sprintf ("%.8f", $x0 / $y0 ) );
+
         splice(@$x,1);			# keep single element
         return $x;
         }
@@ -950,7 +986,7 @@ sub _digit
   
   my $elem = int($n / $BASE_LEN);	# which array element
   my $digit = $n % $BASE_LEN;		# which digit in this element
-  $elem = '0000000'.@$x[$elem];		# get element padded with 0's
+  $elem = '0' x $BASE_LEN . @$x[$elem];	# get element padded with 0's
   substr($elem,-$digit-1,1);
   }
 
