@@ -8,22 +8,29 @@ require Exporter;
 use vars qw/@ISA $VERSION/;
 @ISA = qw(Exporter);
 
-$VERSION = '0.02';
+$VERSION = '0.04';
+
+sub api_version () { 1; }
 
 # Package to to test Bigint's simulation of Calc
 
 # uses Calc, but only features the strictly necc. methods.
 
-use Math::BigInt::Calc '0.33';
+use Math::BigInt::Calc '0.40';
 
 BEGIN
   {
   no strict 'refs';
-  foreach (qw/	base_len new zero one two copy str num add sub mul div inc dec
-		acmp len digit zeros
-		is_zero is_one is_odd is_even is_one check
-		to_small to_large
-		/)
+  foreach (qw/	
+	base_len new zero one two ten copy str num add sub mul div mod inc dec
+	acmp len digit zeros
+	rsft lsft
+	fac pow gcd log_int sqrt root
+	is_zero is_one is_odd is_even is_one is_two is_ten check
+	as_hex as_bin from_hex from_bin	
+	modpow modinv
+	and xor or
+	/)
     {
     my $name  = "Math::BigInt::Calc::_$_";
     *{"Math::BigInt::BareCalc::_$_"} = \&$name;
